#!perl

package PureMessage::Reports::OS::HPUX;

use strict;
use warnings;
use PureMessage::Reports::OS;
use base qw(PureMessage::Reports::OS);

use PureMessage::Reports::OS::Pipeline::ReEntrant;
use PureMessage::Reports::OS::Pipeline::Continuous;

our(@tags) = qw(parse_df parse_vmstat parse_iostat parse_uptime);

return 1;

sub new {
    my($class, %args) = (@_);
    my $self = $class->SUPER::new(disc_read_x => 0, disc_write_x => 0, %args);

    bless $self, $class;
    
    if(!$self->{pipelines})
    {
        $self->{pipelines} = [
            $self->pipeline_uptime,
            
            PureMessage::Reports::OS::Pipeline::ReEntrant->new(
                program => [ "/usr/bin/df", "-Pkl" ],
                callback_exit => sub { $self->parse_df(@_); },
                delay => $self->{delay},
            ),
        
            PureMessage::Reports::OS::Pipeline::Continuous->new(
                program => [ "/usr/bin/vmstat", $self->{step} ],
                callback => sub { $self->parse_vmstat(@_); },
                # first line is "summary" since boot which we want to avoid
                preamble => qr{[0-9]\n},
                eod => qr{\n},
            ),
            
            PureMessage::Reports::OS::Pipeline::Continuous->new(
                program => [ "/usr/bin/iostat", $self->{step} ],
                callback => sub { $self->parse_iostat(@_); },
                eod => qr{\n\n},
            ),
        ];
    }

    $self->{tags} ||= \@tags;
    $self->{swap_free} = 0;
    
    $self;
}

sub parse_vmstat {
    my($self, $pipe, $data) = @_;
    
    $data =~ s{^\s+}{};

    return if($data =~ m{^[a-z]});

    my(@data) = (split(/\s+/, $data));

    if(defined($data[17])) {
        $self->{cpu_idle_x} += $data[17];
        $self->{ram_free_x} += $data[4];
        $self->{vmstat_rows_in} ++;

        if($self->stat_ready("vmstat")) {
            $self->{cpu_free} = $self->{cpu_idle_x} / $self->{sample_average}; # /
            $self->{cpu_idle_x} = 0;
            $self->{ram_free} = $self->{ram_free_x} / $self->{sample_average}; # /
            $self->{ram_free_x} = 0;
        }
    }
    else
    {
        warn "malformed vmstat line \"$data\"";
    }
    
}

sub parse_iostat {
    my($self, $pipe, $data) = @_;
    
    my(@data) = split(/\n/, $data);
    my $did;
    
    foreach my $line (@data) {
        $line =~ s{^\s+}{};
        my(@line) = split(/\s+/, $line);
        next if((!@line) || $line[0] eq "device");
        if(defined($line[3])) {
            $did++;
            $self->{disc_read} += ($line[1] * $self->{step}) / 2;
            $self->{disc_write} += ($line[1] * $self->{step}) / 2;
        }
    }
    
    if($did) {
        $self->{iostat_rows_in}++;
        
        if($self->stat_ready("iostat")) {
            $self->{disc_read} = $self->{disc_read_x};
            $self->{disc_write} = $self->{disc_write_x};
            $self->{disc_read_x} = 0;
            $self->{disc_write_x} = 0;
        }
    }
}
