#!perl

package PureMessage::Reports::OS::Linux;

use strict;
use warnings;
use PureMessage::Reports::OS;
use base qw(PureMessage::Reports::OS);

use PureMessage::Reports::OS::Pipeline::ReEntrant;
use PureMessage::Reports::OS::Pipeline::Continuous;

our(@tags) = qw(parse_uptime parse_df parse_vmstat parse_free);

return 1;

sub new {
    my($class, %args) = (@_);
    my $self = $class->SUPER::new(disc_read_x => 0, disc_write_x => 0, %args);

    bless $self, $class;
    
    if(!$self->{pipelines}) {
        $self->{pipelines} = [
            $self->pipeline_uptime,
            $self->pipeline_df,
        
            PureMessage::Reports::OS::Pipeline::Continuous->new(
                program => [ "/usr/bin/vmstat", "-n", $self->{step} ],
                callback => sub { $self->parse_vmstat(@_); },
                # first line is "summary" since boot which we want to avoid
                preamble => qr{[0-9]\n},
                eod => qr{\n},
            ),
                
            PureMessage::Reports::OS::Pipeline::Continuous->new(
                program => [ "/usr/bin/free", "-s", $self->{step} ],
                callback => sub { $self->parse_free(@_); },
                eod => qr{\n\n},
            ),
        ];
    }

    $self->{tags} ||= \@tags;
    
    $self;
}

sub parse_free {
    my($self, $pipe, $data) = @_;
    
    if($data =~ m{-/+.+\s([0-9]+)\nSwap:.+\s([0-9]+)\n}s) {
        $self->{ram_free_x} += $1;
        $self->{swap_free_x} += $2;
        $self->{free_rows_in}++;

        if($self->stat_ready("free")) {
            $self->{ram_free} = $self->{ram_free_x} / $self->{sample_average}; # /
            $self->{swap_free} = $self->{swap_free_x} / $self->{sample_average}; # /
            $self->{ram_free_x} = 0;
            $self->{swap_free_x} = 0;
        }
    } else {
        warn "malformed free line \"$data\"";
    }
}

sub parse_vmstat {
    my($self, $pipe, $data) = @_;
    
    $data =~ s{^\s+}{};
    my(@data) = (split(/\s+/, $data));
    if(defined($data[14])) {
        $self->{disc_read_x} += $data[6] + $data[8];
        $self->{disc_write_x} += $data[7] + $data[9];
        $self->{cpu_idle_x} += $data[14];
        $self->{vmstat_rows_in} ++;


        if($self->stat_ready("vmstat")) {
            $self->{disc_read} = $self->{disc_read_x};
            $self->{disc_write} = $self->{disc_write_x};
            $self->{cpu_free} = $self->{cpu_idle_x} / $self->{sample_average}; # /
            
            map { $self->{$_} = 0 }
                qw(disc_read_x disc_write_x cpu_idle_x);
        }
    } else {
        warn "malformed vmstat line \"$data\"";
    }
    
}
