#!perl
# PureMessage OS Health Reports
# "Pipeline" base object

package PureMessage::Reports::OS::Pipeline;

use strict;
use warnings;

use IPC::Run qw(harness);

return 1;

# Standard pipeline initialisation attributes:
# program - anonymous array of command-line to execute
#  eg; [ "/usr/bin/df", "-k" ]
#
# callback_exit - coderef to run when process exits
#
#

sub new {
    my $class = shift;
    my $self = bless { @_ }, $class;
 
    return $self;
}

sub start {
    my $self = shift;
    
    my $harness = harness($self->{program}, ">", sub { $self->add_data(@_) });
    if($harness->start()) {
        $self->{ipc} = $harness;
        return $self->{ipc};
    } else {
        warn join(" ", @_), ": $!";
        return;
    }
}

sub stop {
    my $self = shift;
    if($self->{ipc}) {
        $self->{ipc}->kill_kill;
        delete($self->{ipc});
    }
	
	delete($self->{preambled});
	
}

sub check {
    my $self = shift;
    
    if($self->{ipc}->pumpable) {
        $self->{ipc}->pump_nb;
        if($self->{ipc}->pumpable) {
            return 1;
        }
        else
        {
            return($self->check(@_));
        }
    } else {
        $self->stop;
        if($self->{callback_exit}) {
            return $self->{callback_exit}->($self);
        } else {
			warn "dubious: pipeline ", join(" ", @{$self->{program}}),
				" exited but is not supposed to.";
            return 0;
        }
    }
}

sub add_data {
    my $self = shift;
    $self->{data} .= join("", @_);
}

sub data {
    my $self = shift;
    if($self->{data}) {
        $self->{old_data} = $self->{data};
        $self->{data} = "";
        return $self->{old_data};
    } else {
        return "";
    }
}
