#!perl
# PureMessage OS Health Reports
# Continuous Feed pipeline object

package PureMessage::Reports::OS::Pipeline::Continuous;

use strict;
use warnings;

use PureMessage::Reports::OS::Pipeline;
use base qw(PureMessage::Reports::OS::Pipeline);

# Extra attributes to configure ::Continuous:
#  eod - Required - End Of Datum - Regular expression used to signify
#        that data should be
#        returned. Data returned includes the eod.
#
#  callback - Required - callback to run when more data is available
#
#  preamble - IF set, no data will be returned until this pattern is matched.
#
#
#
# Attributes used internally:
#  preambled - Set once {preamble} is matched.
#
#  row - Incremented each time {eod} is matched.
#

return 1;

sub add_data {
    my $self = shift;
    my $row = 0;
    $self->SUPER::add_data(@_);
        
    if(
        $self->{preamble} && !$self->{preambled} &&
        $self->{data} =~ s{^(.*?$self->{preamble})}{}s) {
        $self->{preambled}++;
    }
    
    if($self->{preambled} || !$self->{preamble}) {
        while($self->{data} =~ s{^(.*?$self->{eod})}{}s) {
            $self->{row}++;
            $row++;
            $self->{callback}->($self, $1);
        }
        
        return $row;
    }
    
    return;
}
