#!perl

package PureMessage::Reports::OS::SunOS;

use strict;
use warnings;
use PureMessage::Reports::OS;
use base qw(PureMessage::Reports::OS);

use PureMessage::Reports::OS::Pipeline::ReEntrant;
use PureMessage::Reports::OS::Pipeline::Continuous;

our(@tags) = qw(parse_uptime parse_df parse_vmstat parse_iostat);

return 1;

sub new {
    my($class, %args) = (@_);
    my $self = $class->SUPER::new(disc_read_x => 0, disc_write_x => 0, %args);

    bless $self, $class;
    
    if(!$self->{pipelines}) {
        $self->{pipelines} = [
            $self->pipeline_uptime,
            $self->pipeline_df,
                
            PureMessage::Reports::OS::Pipeline::Continuous->new(
                program => [ "/usr/bin/vmstat", $self->{step} ],
                callback => sub { $self->parse_vmstat(@_); },
                # first line is "summary" since boot which we want to avoid
                preamble => qr{[0-9]\n},
                eod => qr{\n},
            ),
                
            PureMessage::Reports::OS::Pipeline::Continuous->new(
                program => [ "/usr/bin/iostat", "-xtc", $self->{step} ],
                callback => sub { $self->parse_iostat(@_); },
                # first line is "summary" since boot which we want to avoid
                preamble => qr{\n\s},
                eod => qr{\n\s},
            ),
                
        ];
    }
    
    $self->{tags} ||= \@tags;
    
    $self;
}

sub parse_vmstat {
    my($self, $pipe, $data) = @_;
    
    return unless($data =~ m{^\s+[0-9]});
    $data =~ s{^\s+}{};
    
    # SunOS repeats it's headers every so often...
    
    my(@data) = (split(/\s+/, $data));
    if(defined($data[21])) {
        $self->{ram_free_x} += $data[4];
        $self->{swap_free_x} += $data[3];
        $self->{cpu_free_x} += $data[21];
        $self->{vmstat_rows_in} ++;

        if($self->stat_ready("vmstat")) {
            foreach (qw(ram_free swap_free cpu_free)) {
                $self->{$_} = $self->{$_ . "_x" } / $self->{sample_average}; # /
                $self->{$_ . "_x"} = 0;
            }
        }
    } else {
        warn "malformed vmstat line \"$data\"";
    }
    
}

sub parse_iostat {
    my($self, $pipe, $data) = @_;
    
    $data =~ s{^\s+}{};
    
    my(@data) = (split(/\n/, $data));
    splice(@data, 0, 2);
    
    my($kps_in, $kps_out);
    
    while(my $line = shift(@data)) {
        my(@line) = split(/\s+/, $line);
        
        if(defined($line[4])) {
            # skip NFS; they're network, not disc
            if(substr($line[0], 0, 3) ne "nfs") {
                $self->{disc_read_x} += $line[3] * $self->{step};
                $self->{disc_write_x} += $line[4] * $self->{step};
            }
        } elsif($line ne " ") { # gunk leftover from eod
            warn "malformed iostat line \"$line\"";
        }
    }
    
    $self->{iostat_rows_in}++;
    
    if($self->stat_ready("iostat")) {
        $self->{disc_read} = $self->{disc_read_x};
        $self->{disc_write} = $self->{disc_write_x};
        $self->{disc_read_x} = 0;
        $self->{disc_write_x} = 0;
    }
}
